import os
import shutil
import json

from tools import make_bundle
# prepare script with standard distribution
www = os.path.join(os.path.dirname(os.path.dirname(os.getcwd())), 'www')
folders = [
    os.path.join(www, 'src', 'Lib'),
    os.path.join(www, 'src', 'libs')
]

res = {}
for folder in folders:
    res.update(make_bundle.bundle(folder))

# copy stdlib in brython_stdlib.js
# and initialise brython_modules.js with the same content

bundle_names = ['brython_stdlib.js', 'brython_modules.js']
for name in bundle_names:
    with open(name, 'w', encoding='utf-8') as out:
        out.write('__BRYTHON__.use_VFS = true;\n')
        out.write('__BRYTHON__.VFS = {}\n'.format(json.dumps(res)))

# copy brython.js
shutil.copyfile(os.path.join(www, 'src', 'brython.js'), 'brython.js')

# get version info from www/src/version_info.js, generated by make_script
info_path = os.path.join(www, "src", "version_info.js")
with open(info_path, encoding="utf-8") as fobj:
    info = fobj.read()
class A:
    pass
ns = {"__BRYTHON__":A}
exec(info, ns)
version_info = ns["__BRYTHON__"].version_info
version = ".".join(str(x) for x in version_info[:3])

# create zip files
import tarfile
import zipfile

name = 'Brython-%s' %version
dest_path = name
dist1 = tarfile.open(dest_path + '.tar.gz', mode='w:gz')
dist2 = tarfile.open(dest_path+'.tar.bz2', mode='w:bz2')
dist3 = zipfile.ZipFile(dest_path + '.zip', mode='w',
                        compression=zipfile.ZIP_DEFLATED)

paths = ['index.html', 'brython.js', 'brython_stdlib.js', 'brython_modules.js']

for arc, wfunc in (dist1, dist1.add), (dist2, dist2.add), (dist3, dist3.write):
    for path in paths:
        wfunc(path, arcname=os.path.join(name, path))

    arc.close()



# -*- coding: utf-8 -*-
#

"""Simple HTTP Server.
Supports browser cache and HTTP compression."""

import os
import sys
import datetime
import io

import email
import urllib.parse
from http import HTTPStatus
import http.cookiejar
import http.server as server
from http.server import SimpleHTTPRequestHandler

# Python might be built without gzip / zlib
try:
    import zlib
except ImportError:
    zlib = None

# List of commonly compressed content types, copied from
# https://github.com/h5bp/server-configs-apache.
# compressed_types is set to this list when the server is started with
# command line option --gzip.
commonly_compressed_types = [ "application/atom+xml",
                              "application/javascript",
                              "application/json",
                              "application/ld+json",
                              "application/manifest+json",
                              "application/rdf+xml",
                              "application/rss+xml",
                              "application/schema+json",
                              "application/vnd.geo+json",
                              "application/vnd.ms-fontobject",
                              "application/x-font-ttf",
                              "application/x-javascript",
                              "application/x-web-app-manifest+json",
                              "application/xhtml+xml",
                              "application/xml",
                              "font/eot",
                              "font/opentype",
                              "image/bmp",
                              "image/svg+xml",
                              "image/vnd.microsoft.icon",
                              "image/x-icon",
                              "text/cache-manifest",
                              "text/css",
                              "text/html",
                              "text/javascript",
                              "text/plain",
                              "text/vcard",
                              "text/vnd.rim.location.xloc",
                              "text/vtt",
                              "text/x-component",
                              "text/x-cross-domain-policy",
                              "text/xml"
                            ]

# Generators for HTTP compression

# Generators for HTTP compression

def _zlib_producer(fileobj, wbits):
    """Generator that yields data read from the file object fileobj,
    compressed with the zlib library.
    wbits is the same argument as for zlib.compressobj.
    """
    bufsize = 2 << 17
    producer = zlib.compressobj(wbits=wbits)
    with fileobj:
        while True:
            buf = fileobj.read(bufsize)
            if not buf: # end of file
                yield producer.flush()
                return
            yield producer.compress(buf)

def _gzip_producer(fileobj):
    """Generator for gzip compression."""
    return _zlib_producer(fileobj, 25)

def _deflate_producer(fileobj):
    """Generator for deflage compression."""
    return _zlib_producer(fileobj, 15)

class RequestHandler(SimpleHTTPRequestHandler):

    # List of Content Types that are returned with HTTP compression.
    # Set to the empty list by default (no compression).
    compressed_types = []

    # Dictionary mapping an encoding (in an Accept-Encoding header) to a
    # generator of compressed data. By default, provided zlib is available,
    # the supported encodings are gzip and deflate.
    # Override if a subclass wants to use other compression algorithms.
    compressions = {}
    if zlib:
        compressions = {
            'deflate': _deflate_producer,
            'gzip': _gzip_producer,
            'x-gzip': _gzip_producer # alias for gzip
    }

    def _make_chunk(self, data):
        """Produces a data chunk for Chunked Transfer Encoding."""
        return ("{:X}".format(len(data)).encode("ascii") + b"\r\n" + data
            + b"\r\n")

    def do_GET(self):
        """Serve a GET request."""
        f = self.send_head()
        if f:
            try:
                if hasattr(f, "read"):
                    self.copyfile(f, self.wfile)
                else:
                    # Generator for compressed data
                    if self.protocol_version >= "HTTP/1.1":
                        # Chunked Transfer
                        for data in f:
                            if data:
                                self.wfile.write(self._make_chunk(data))
                        self.wfile.write(self._make_chunk(b''))
                    else:
                        for data in f:
                            self.wfile.write(data)
            finally:
                f.close()

    def send_head(self):
        """Common code for GET and HEAD commands.

        This sends the response code and MIME headers.

        Return value is either a file object (which has to be copied
        to the outputfile by the caller unless the command was HEAD,
        and must be closed by the caller under all circumstances), or
        None, in which case the caller has nothing further to do.

        """
        path = self.translate_path(self.path)
        f = None
        if os.path.isdir(path):
            parts = urllib.parse.urlsplit(self.path)
            if not parts.path.endswith('/'):
                # redirect browser - doing basically what apache does
                self.send_response(HTTPStatus.MOVED_PERMANENTLY)
                new_parts = (parts[0], parts[1], parts[2] + '/',
                             parts[3], parts[4])
                new_url = urllib.parse.urlunsplit(new_parts)
                self.send_header("Location", new_url)
                self.end_headers()
                return None
            for index in "index.html", "index.htm":
                index = os.path.join(path, index)
                if os.path.exists(index):
                    path = index
                    break
            else:
                return self.list_directory(path)
        ctype = self.guess_type(path)
        try:
            f = open(path, 'rb')
        except OSError:
            self.send_error(HTTPStatus.NOT_FOUND, "File not found")
            return None

        try:
            fs = os.fstat(f.fileno())
            content_length = fs[6]
            # Use browser cache if possible
            if ("If-Modified-Since" in self.headers
                    and "If-None-Match" not in self.headers):
                # compare If-Modified-Since and time of last file modification
                try:
                    ims = email.utils.parsedate_to_datetime(
                        self.headers["If-Modified-Since"])
                except (TypeError, IndexError, OverflowError, ValueError):
                    # ignore ill-formed values
                    pass
                else:
                    if ims.tzinfo is None:
                        # obsolete format with no timezone, cf.
                        # https://tools.ietf.org/html/rfc7231#section-7.1.1.1
                        ims = ims.replace(tzinfo=datetime.timezone.utc)
                    if ims.tzinfo is datetime.timezone.utc:
                        # compare to UTC datetime of last modification
                        last_modif = datetime.datetime.fromtimestamp(
                            fs.st_mtime, datetime.timezone.utc)
                        # remove microseconds, like in If-Modified-Since
                        last_modif = last_modif.replace(microsecond=0)

                        if last_modif <= ims:
                            self.send_response(HTTPStatus.NOT_MODIFIED)
                            self.end_headers()
                            f.close()
                            return None

            self.send_response(HTTPStatus.OK)
            self.send_header("Content-type", ctype)
            self.send_header("Last-Modified",
                self.date_time_string(fs.st_mtime))

            # Use HTTP compression if possible

            if ctype not in self.compressed_types:
                self.send_header("Content-Length", str(content_length))
                self.end_headers()
                return f

            # Get accepted encodings ; "encodings" is a dictionary mapping
            # encodings to their quality ; eg for header "gzip; q=0.8",
            # encodings["gzip"] is set to 0.8
            accept_encoding = self.headers.get_all("Accept-Encoding", ())
            encodings = {}
            for accept in http.cookiejar.split_header_words(accept_encoding):
                params = iter(accept)
                encoding = next(params, ("", ""))[0]
                quality, value = next(params, ("", ""))
                if quality == "q" and value:
                    try:
                        q = float(value)
                    except ValueError:
                        # Invalid quality : ignore encoding
                        q = 0
                else:
                    q = 1 # quality defaults to 1
                if q:
                    encodings[encoding] = max(encodings.get(encoding, 0), q)

            compressions = set(encodings).intersection(self.compressions)
            compression = None
            if compressions:
                # Take the encoding with highest quality
                compression = max((encodings[enc], enc)
                    for enc in compressions)[1]
            elif '*' in encodings and self.compressions:
                # If no specified encoding is supported but "*" is accepted,
                # take one of the available compressions.
                compression = list(self.compressions)[0]
            if compression:
                # If at least one encoding is accepted, send data compressed
                # with the selected compression algorithm.
                producer = self.compressions[compression]
                self.send_header("Content-Encoding", compression)
                if content_length < 2 << 18:
                    # For small files, load content in memory
                    with f:
                        content = b''.join(producer(f))
                    content_length = len(content)
                    f = io.BytesIO(content)
                else:
                    chunked = self.protocol_version >= "HTTP/1.1"
                    if chunked:
                        # Use Chunked Transfer Encoding (RFC 7230 section 4.1)
                        self.send_header("Transfer-Encoding", "chunked")
                    self.end_headers()
                    # Return a generator of pieces of compressed data
                    return producer(f)

            self.send_header("Content-Length", str(content_length))
            self.end_headers()
            return f
        except:
            f.close()
            raise

def run(port=8080):
    server_address, handler = ('', port), RequestHandler
    handler.compressed_types = commonly_compressed_types
    httpd = server.HTTPServer(server_address, handler)

    print(("Server running on port http://localhost:{}.".format(port)))
    print("Press CTRL+C to Quit.")
    httpd.serve_forever()

if __name__ == "__main__":
    run()


"""Detect all Python scripts in HTML pages in current folder and subfolders.
Generate brython_modules.js, a bundle with all the modules and packages used
by an application.
Generate a Python package ready for installation and upload on PyPI.
"""

import os
import shutil
import html.parser
import ast
import json

# Template for application setup.py script
setup = """from setuptools import setup, find_packages

import os

if os.path.exists('README.rst'):
    with open('README.rst', encoding='utf-8') as fobj:
        LONG_DESCRIPTION = fobj.read()

setup(
    name='{app_name}',
    version='{version}',

    # The project's main homepage.
    url='{url}',

    # Author details
    author='{author}',
    author_email='{author_email}',

    # License
    license='{license}',

    packages=['data'],
    py_modules=["{app_name}"],
    package_data={{'data':[{files}]}}
)
"""

# Template for the application script
app = """import os
import shutil
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('--install',
    help='Install {app_name} in an empty directory',
    action="store_true")
args = parser.parse_args()

files = ({files})

if args.install:
    print('Installing {app_name} in an empty directory')

    src_path = os.path.join(os.path.dirname(__file__), 'data')

    if os.listdir(os.getcwd()):
        print('{app_name} can only be installed in an empty folder')
        import sys
        sys.exit()

    for path in files:
        shutil.copyfile(os.path.join(src_path, path), path)

"""

# Get all modules in the Brython standard distribution.
# They must be in brython_stdlib.js somewhere in the current directory
# or below.
print('searching brython_stdlib.js...')
stdlib = {}
stdlib_dir = None
for dirname, dirnames, filenames in os.walk(os.getcwd()):
    for filename in filenames:
        if filename == "brython_stdlib.js":
            stdlib_dir = dirname
            path = os.path.join(dirname, filename)
            with open(path, encoding="utf-8") as fobj:
                modules = fobj.read()
                modules = modules[modules.find('{'):]
                stdlib = json.loads(modules)


# get all Python modules and packages
user_modules = {}

packages = set()

def is_package(folder):
    """Test if folder is a package, ie has __init__.py and all the folders
    above until os.getcwd() also have __init__.py.
    Use set "packages" to cache results.
    """
    if folder in packages:
        return True
    current = folder
    while True:
        if not os.path.exists(os.path.join(current, "__init__.py")):
            return False
        current = os.path.dirname(current)
        if current == os.getcwd():
            packages.add(folder)
            return True

print('finding packages...')
for dirname, dirnames, filenames in os.walk(os.getcwd()):
    for filename in filenames:
        name, ext = os.path.splitext(filename)
        if not ext == ".py":
            continue
        if dirname == os.getcwd():
            # modules in the same directory
            path = os.path.join(dirname, filename)
            with open(path, encoding="utf-8") as fobj:
                user_modules[name] = [ext, fobj.read()]
        elif is_package(dirname):
            # modules in packages below current directory
            path = os.path.join(dirname, filename)
            package = dirname[len(os.getcwd()) + 1:].replace(os.sep, '.')
            if filename == "__init__.py":
                module_name = package
            else:
                module_name = "{}.{}".format(package, name)
            with open(path, encoding="utf-8") as fobj:
                user_modules[module_name] = [ext, fobj.read()]
                if module_name == package:
                    user_modules[module_name].append(1)


class CharsetDetector(html.parser.HTMLParser):
    """Used to detect <meta charset="..."> in HTML page."""

    def __init__(self, *args, **kw):
        kw.setdefault('convert_charrefs', True)
        try:
            html.parser.HTMLParser.__init__(self, *args, **kw)
        except TypeError:
            # convert_charrefs is only supported by Python 3.4+
            del kw['convert_charrefs']
            html.parser.HTMLParser.__init__(self, *args, **kw)

        self.encoding = "iso-8859-1"

    def handle_starttag(self, tag, attrs):
        if tag.lower() == "meta":
            for key, value in attrs:
                if key == "charset":
                    self.encoding = value

class BrythonScriptsExtractor(html.parser.HTMLParser):
    """Used to extract all Brython scripts in HTML pages."""

    def __init__(self, dirname, **kw):
        kw.setdefault('convert_charrefs', True)
        try:
            html.parser.HTMLParser.__init__(self, **kw)
        except TypeError:
            # convert_charrefs is only supported by Python 3.4+
            del kw['convert_charrefs']
            html.parser.HTMLParser.__init__(self, **kw)

        self.dirname = dirname
        self.scripts = []
        self.py_tags = [] # stack of Python blocks
        self.tag_stack = []

    def handle_starttag(self, tag, attrs):
        if tag.lower() == "script":
            _type = "js_script"
            src = None
            for key, value in attrs:
                if key == 'type' and value == "text/python":
                    _type = "py_script"
                elif key == "src":
                    src = value
            if _type == "py_script" and src:
                _type = "py_script_with_src"
                path = os.path.join(self.dirname, src)
                with open(path, encoding="utf-8") as fobj:
                    self.scripts.append(fobj.read())
            self.tag_stack.append(_type)

    def handle_endtag(self, tag):
        if tag.lower() == "script":
            self.tag_stack.pop()

    def handle_data(self, data):
        """Data is printed unchanged"""
        if data.strip():
            if self.tag_stack and self.tag_stack[-1].lower() == "py_script":
                self.scripts.append(data)


class VFSReplacementParser(html.parser.HTMLParser):
    """Used to replace brython_stdlib.js by brython_modules.js in HTML
    pages."""

    def __init__(self, path, **kw):
        kw.setdefault('convert_charrefs', True)
        try:
            html.parser.HTMLParser.__init__(self, **kw)
        except TypeError:
            # convert_charrefs is only supported by Python 3.4+
            del kw['convert_charrefs']
            html.parser.HTMLParser.__init__(self, **kw)
        self.vfs = False
        self.has_vfs = False

    def handle_starttag(self, tag, attrs):
        if tag.lower() == "script":
            _type = "js_script"
            src = None
            for key, value in attrs:
                if key == "src":
                    elts = value.split("/")
                    if elts and elts[-1] == "brython_stdlib.js":
                        self.vfs = True
                        self.has_vfs = True
                        self.attrs = attrs
                        self.start = self.getpos()
                        return
        self.vfs = False

    def handle_endtag(self, tag):
        if tag.lower() == "script" and self.vfs:
            self.end = self.getpos()

class ImportsFinder(ast.NodeVisitor):
    """Used to detect all imports in an AST tree and store the results in
    attribute imports.
    """

    def __init__(self, *args, **kw):
        self.package = kw.pop("package") or ""
        ast.NodeVisitor.__init__(self, *args, **kw)
        self.imports = set()

    def visit_Import(self, node):
        for name in node.names:
            if name.name != '*':
                # For "import A.B.C", modules A, A.B and A.B.C must be stored
                # in self.imports
                elts = name.name.split('.')
                while elts:
                    self.imports.add('.'.join(elts))
                    elts.pop()

    def visit_ImportFrom(self, node):
        package = self.package.split('.')
        for _ in range(node.level - 1):
            package.pop()
        package = '.'.join(package)
        if package:
            package += '.'
        if node.module is None:
            # syntax "from .. import name1, name2"
            self.imports.add(package)
            for name in node.names:
                if name.name != '*':
                    self.imports.add('{}{}'.format(package, name.name))
        else:
            # syntax "from .foo import name1, name2"
            self.imports.add('{}{}'.format(package, node.module))
            for name in node.names:
                if name.name != "*":
                    self.imports.add('{}{}.{}'.format(package, node.module,
                        name.name))

class ModulesFinder:

    def __init__(self, directory=os.getcwd()):
        self.directory = directory
        self.modules = set()

    def get_imports(self, src, package=None):
        """Get all imports in source code src."""
        tree = ast.parse(src)
        finder = ImportsFinder(package=package)
        finder.visit(tree)
        for module in finder.imports:
            if module in self.modules:
                continue
            for module_dict in [stdlib, user_modules]:
                if module in module_dict:
                    self.modules.add(module)
                    if module_dict[module][0] == '.py':
                        imports = self.get_imports(module_dict[module][1])

        return finder.imports

    def norm_indent(self, script):
        """Scripts in Brython page may start with an indent, remove it before
        building the AST.
        """
        indent = None
        lines = []
        for line in script.split('\n'):
            if line.strip() and indent is None:
                indent = len(line) - len(line.lstrip())
                line = line[indent:]
            elif indent is not None:
                line = line[indent:]
            lines.append(line)
        return '\n'.join(lines)

    def inspect(self):
        """Walk the directory to find all pages with Brython scripts, parse
        them to get the list of modules needed to make them run.
        """
        for dirname, dirnames, filenames in os.walk(self.directory):
            for name in dirnames:
                if name.endswith('__dist__'):
                    # don't inspect files in the subfolder __dist__
                    dirnames.remove(name)
                    break
            for filename in filenames:
                path = os.path.join(dirname, filename)
                if path == __file__:
                    continue
                ext = os.path.splitext(filename)[1]
                if ext.lower() == '.html':
                    # detect charset
                    charset_detector = CharsetDetector()
                    with open(path, encoding="iso-8859-1") as fobj:
                        charset_detector.feed(fobj.read())

                    # get text/python scripts
                    parser = BrythonScriptsExtractor(dirname)
                    with open(path, encoding=charset_detector.encoding) as fobj:
                        parser.feed(fobj.read())
                    for script in parser.scripts:
                        script = self.norm_indent(script)
                        try:
                            self.get_imports(script)
                        except SyntaxError:
                            print('syntax error', path)
                elif ext.lower() == '.py':
                    if dirname != self.directory and not is_package(dirname):
                        continue
                    # get package name
                    package = dirname[len(self.directory) + 1:] or None
                    with open(path, encoding="utf-8") as fobj:
                        try:
                            self.get_imports(fobj.read(), package)
                        except SyntaxError:
                            print('syntax error', path)

    def make_brython_modules(self):
        """Build brython_modules.js from the list of modules needed by the
        application.
        """
        vfs = {}
        for module in self.modules:
            if module in stdlib:
                vfs[module] = stdlib[module]
            else:
                vfs[module] = user_modules[module]
                elts = module.split('.')
                for i in range(1, len(elts)):
                    pkg = '.'.join(elts[:i])
                    if not pkg in vfs:
                        vfs[pkg] = user_modules[pkg]

        # save in brython_modules.js
        path = os.path.join(stdlib_dir, "brython_modules.js")
        with open(path, "w", encoding="utf-8") as out:
            out.write("__BRYTHON__.use_VFS = true\n__BRYTHON__.VFS = ")
            json.dump(vfs, out)

    def _dest(self, base_dir, dirname, filename):
        """Build the destination path for a file."""
        elts = dirname[len(os.getcwd()) + 1:].split(os.sep)
        dest_dir = base_dir
        for elt in elts:
            dest_dir = os.path.join(dest_dir, elt)
            if not os.path.exists(dest_dir):
                os.mkdir(dest_dir)
        return os.path.join(dest_dir, filename)

    def make_setup(self):
        """Make the setup script (setup.py) and the entry point script
        for the application."""
        # Create a temporary directory
        temp_dir = '__dist__'
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)
        os.mkdir(temp_dir)

        # Create a package "data" in this directory
        data_dir = os.path.join(temp_dir, 'data')
        os.mkdir(data_dir)
        with open(os.path.join(data_dir, "__init__.py"), "w") as out:
            out.write('')

        # If there is a brython_setup.json file, use it to get information
        if os.path.exists("brython_setup.json"):
            with open("brython_setup.json", encoding="utf-8") as fobj:
                info = json.load(fobj)
        else:
            # Otherwise, ask setup information
            while True:
                app_name = input("Application name: ")
                if app_name:
                    break
            while True:
                version = input("Version: ")
                if version:
                    break
            author = input("Author: ")
            author_email = input("Author email: ")
            license = input("License: ")
            url = input("Project url: ")
            info = {
                "app_name": app_name,
                "version": version,
                "author": author,
                "author_email": author_email,
                "license": license,
                "url": url
            }
            # Store information in brython_setup.json
            with open("brython_setup.json", "w", encoding="utf-8") as out:
                json.dump(info, out, indent=4)

        # Store all application files in the temporary directory. In HTML
        # pages, replace "brython_stdlib.js" by "brython_modules.js"
        files = []
        for dirname, dirnames, filenames in os.walk(self.directory):
            if dirname == "__dist__":
                continue
            if "__dist__" in dirnames:
                dirnames.remove("__dist__")
            for filename in filenames:
                path = os.path.join(dirname, filename)
                files.append(path[len(os.getcwd()) + 1:])
                if os.path.splitext(filename)[1] == '.html':
                    # detect charset
                    charset_detector = CharsetDetector()
                    with open(path, encoding="iso-8859-1") as fobj:
                        charset_detector.feed(fobj.read())
                    encoding = charset_detector.encoding

                    # get text/python scripts
                    parser = VFSReplacementParser(dirname)
                    with open(path, encoding=encoding) as fobj:
                        parser.feed(fobj.read())
                    if not parser.has_vfs:
                        # save file
                        dest = self._dest(data_dir, dirname, filename)
                        shutil.copyfile(path, dest)
                        continue
                    with open(path, encoding=encoding) as fobj:
                        lines = fobj.readlines()
                        start_line, start_pos = parser.start
                        end_line, end_pos = parser.end
                        res = ''.join(lines[:start_line - 1])
                        for num in range(start_line - 1, end_line):
                            res += lines[num].replace("brython_stdlib.js",
                                "brython_modules.js")
                        res += ''.join(lines[end_line:])
                    dest = self._dest(data_dir, dirname, filename)
                    with open(dest, 'w', encoding=encoding) as out:
                        out.write(res)
                else:
                    dest = self._dest(data_dir, dirname, filename)
                    shutil.copyfile(path, dest)

        info["files"] = ',\n'.join('"{}"'.format(file) for file in files)

        # Generate setup.py from the template in string setup
        path = os.path.join(temp_dir, "setup.py")
        with open(path, "w", encoding="utf-8") as out:
            out.write(setup.format(**info))

        # Generate the application script from the template in string app
        path = os.path.join(temp_dir, "{}.py".format(info["app_name"]))
        with open(path, "w", encoding="utf-8") as out:
            out.write(app.format(**info))

if __name__ == "__main__":
    ModulesFinder().make_setup()


"""Make a bundle for a Brython page
Packs all modules and packages in the page folder and in site-packages
"""

import os
import json
import fnmatch

from . import python_minifier

def bundle(folder, ignore=None):
    if ignore is None:
        # check if there is an ".bundle-ignore" file
        ignore = []
        ignore_path = os.path.join(folder, '.bundle-ignore')
        if os.path.exists(ignore_path):
            ignore = [line.strip() for line in
                open(ignore_path, encoding='utf-8').readlines()]

    paths = []
    res = {}
    for dirpath, dirnames, filenames in os.walk(folder):
        if '__pycache__' in dirnames:
            dirnames.remove('__pycache__')
        exclude_dir = []
        for dirname in dirnames:
            init_path = os.path.join(dirpath, dirname, '__init__.py')
            if not os.path.exists(init_path):
                exclude_dir.append(dirname)
        for _dir in exclude_dir:
            dirnames.remove(_dir)
        for filename in filenames:
            name, ext = os.path.splitext(filename)
            if ext == '.py':
                fullpath = os.path.join(dirpath, filename)[len(folder)+1:]
                fullpath = fullpath.replace(os.sep, '/')
                for match in ignore:
                    if fnmatch.fnmatch(fullpath, match):
                        break
                else:
                    parts = fullpath.split('/')
                    if parts[-1]=='__init__.py':
                        mod_name = '.'.join(parts[:-1])
                        is_package = True
                        print('package', mod_name)
                    else:
                        parts[-1] = parts[-1].split('.')[0]
                        mod_name = '.'.join(parts)
                        is_package = False
                    with open(os.path.join(dirpath, filename), 
                        encoding="utf-8") as fobj:
                            mini = python_minifier.minify(fobj.read())
                            if is_package:
                                res[mod_name] = [ext, mini, 1]
                            else:
                                res[mod_name] = [ext, mini]
            elif ext == '.js':
                fullpath = os.path.join(dirpath, filename)[len(folder)+1:]
                for match in ignore:
                    if fnmatch.fnmatch(fullpath, match):
                        break
                else:
                    parts = fullpath.split(os.sep)
                    parts[-1] = parts[-1].split('.')[0]
                    mod_name = '.'.join(parts)
                    with open(os.path.join(dirpath, filename), 
                        encoding="utf-8") as fobj:
                            mini = fobj.read()
                            res[mod_name] = [ext, mini]                
    return res

if __name__ == '__main__':
    www = os.path.join(os.path.dirname(os.getcwd()), 'www')
    folders = [
        os.path.join(www, 'src', 'Lib'),
        os.path.join(www, 'src', 'libs')
    ]
    res = {}
    for folder in folders:
        res.update(bundle(folder))
    bundle_path = os.path.join(os.getcwd(), 'lib', 'brython_stdlib.js')
    with open(bundle_path, 'w', encoding='utf-8') as out:
        out.write('__BRYTHON__.VFS = {}\n'.format(json.dumps(res)))
    print(sorted(list(res.keys())))


# Always prefer setuptools over distutils
from setuptools import setup, find_packages

import os
import shutil

with open('README.rst', encoding='utf-8') as fobj:
    LONG_DESCRIPTION = fobj.read()

setup(
    name='brython',

    version='3.5.0',

    description='Brython is an implementation of Python 3 running in the browser',

    long_description = LONG_DESCRIPTION,

    # The project's main homepage.
    url='http://brython.info',

    # Author details
    author='Pierre Quentel',
    author_email='quentel.pierre@orange.fr',

    packages = ['data', 'data.tools'],

    # Choose your license
    license='BSD',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Interpreters',

        'Operating System :: OS Independent',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
    ],

    # What does your project relate to?
    keywords='Python browser',

    py_modules=["brython", "list_modules", "server"],

    package_data={
        'data': [
            'README.txt',
            'demo.html',
            'brython.js',
            'brython_stdlib.js',
            'unicode.txt'
            ]
    }

)


"""This script creates the basic structure of a Brython project in the
current directory.
"""

import os
import shutil
import json
import argparse

parser = argparse.ArgumentParser()

parser.add_argument('--add_package',
    help="Add a CPython package in Lib/site-packages")

parser.add_argument('--install', help='Install Brython in an empty directory',
    action="store_true")

parser.add_argument('--make_dist', help='Make a Python distribution',
    action="store_true")

parser.add_argument('--modules',
    help='Create brython_modules.js with all the modules used by the application',
    action="store_true")

parser.add_argument('--port', help='Port for the built-in server',
    default=8080)

parser.add_argument('--reset', help='Reset brython_modules.js to stdlib',
    action="store_true")

parser.add_argument('--server', help='Start the built-in server',
    action="store_true")

parser.add_argument('--update', help='Update Brython scripts',
    action="store_true")

args = parser.parse_args()

files = ['README.txt', 'demo.html', 'brython.js', 'brython_stdlib.js',
    'unicode.txt']

if args.add_package:
    print('add package {}...'.format(args.add_package))
    package = __import__(args.add_package)
    package_file = os.path.dirname(package.__file__)
    lib_dir = os.path.join(os.getcwd(), 'Lib')
    if not os.path.exists(lib_dir):
        os.mkdir(lib_dir)
    dest_dir = os.path.join(lib_dir, 'site-packages')
    if not os.path.exists(dest_dir):
        os.mkdir(dest_dir)

    if os.path.splitext(package_file)[1] == '.egg':
        import zipfile
        zf = zipfile.ZipFile(package_file)
        for info in zf.infolist():
            if info.filename.startswith(('__pycache__', 'EGG-INFO')):
                continue
            zf.extract(info, dest_dir)
            print('extract', info.filename)
        zf.close()
        print('done')
    elif os.path.isdir(package_file):
        print('copy folder', package_file)
        dest_dir = os.path.join(dest_dir, args.add_package)
        if os.path.exists(dest_dir):
            shutil.rmtree(dest_dir)
        shutil.copytree(package_file, dest_dir)

if args.install:
    print('Installing Brython in an empty directory')

    src_path = os.path.join(os.path.dirname(__file__), 'data')

    if os.listdir(os.getcwd()):
        print('Brython can only be installed in an empty folder')
        import sys
        sys.exit()

    for path in files:
        shutil.copyfile(os.path.join(src_path, path), path)

if args.update:
    print('Update Brython scripts')

    src_path = os.path.join(os.path.dirname(__file__), 'data')

    for path in files:
        shutil.copyfile(os.path.join(src_path, path), path)

if args.reset:
    print('Reset brython_modules.js to standard distribution')
    shutil.copyfile(os.path.join(os.getcwd(), 'brython_stdlib.js'),
        os.path.join(os.getcwd(), 'brython_modules.js'))

if args.server:
    import server
    server.run(int(args.port))

if args.modules:
    print('Create brython_modules.js with all the modules used by the application')
    import list_modules

    finder = list_modules.ModulesFinder()
    finder.inspect()
    finder.make_brython_modules()

if args.make_dist:
    print('Make a Python distribution for the application')
    import list_modules
    finder = list_modules.ModulesFinder()
    finder.inspect()
    finder.make_brython_modules()
    finder.make_setup()
    print('done')